/*
 * Logger.java
 *
 * Created on 16 May 2009
 *
 * Version 1.2
 *
 * Copyright (c) 2009 Kieran Greer
 *
 */

package org.jlog2;


import org.jlog2.channel.*;

import java.util.*;
import org.jlog2.util.CollectionHandler;


/**
 * This class stores log channels appropriate for each Logger instance and
 * provides the methods to log messages to those channels.
 * @author Kieran Greer
 */
public class Logger
{
    /** Separator to separate the different mesage parts */
    private static final String SEP = ": ";


    /** True if log debug trace as well */
    private boolean debug;

    /** The classname of the class that uses the logger */
    private String className;

    /** The unique id to identify this logger, for example the related classname */
    private String loggerID;
    
    /** The list of channels to write the log messages to */
    private Vector logChannels;

    /** The list of channels to write the log messages to */
    private Vector debugChannels;

    /** The highest level to log messages at */
    private String logLevel;

    /** The highest level to log debug errors at */
    private String debugLevel;


    /**
     * Create a new instance of Logger.
     * @param thisLoggerID the unique logger id.
     */
    public Logger(String thisLoggerID)
    {
        loggerID = thisLoggerID;
        className = thisLoggerID;
        initialiseValues();
    }

    /**
     * Create a new instance of Logger.
     * @param thisLoggerID the unique logger id.
     * @param thisClassName the classname of the class that uses this logger if
     * different from the logger id.
     */
    public Logger(String thisLoggerID, String thisClassName)
    {
        loggerID = thisLoggerID;
        className = thisClassName;
        initialiseValues();
    }

    /** Initialise some values */
    private void initialiseValues()
    {
        debug = false;
        logChannels = null;
        debugChannels = null;
        logLevel = null;
        debugLevel = null;
    }

    /**
     * Set a value indicating if the debug channels should be used.
     * @param thisDebug true if log a debug trace, false otherwise.
     */
    public void setDebug(boolean thisDebug)
    {
        debug = thisDebug;
    }

    /**
     * Get the value indicating if the debug channels are being used.
     * @return the value of debug.
     */
    public boolean getDebug()
    {
        return debug;
    }
    
    /**
     * Get the unique id that created this logger.
     * @return the value of loggerID.
     */
    public String getLoggerID()
    {
        return loggerID;
    }

    /**
     * Add the channels to write the log messages to.
     * @param theLogChannels the channels to write the log messages to.
     */
    public void addLogChannels(Vector theLogChannels)
    {
        if (logChannels == null)
        {
            logChannels = new Vector();
        }

        CollectionHandler.addAll(logChannels, theLogChannels);
    }

    /**
     * Add the channels to write the debug messages to.
     * @param theDebugChannels the channels to write the debug messages to.
     */
    public void addDebugChannels(Vector theDebugChannels)
    {
        if (debugChannels == null)
        {
            debugChannels = new Vector();
        }

        CollectionHandler.addAll(debugChannels, theDebugChannels);
    }

    /**
     * Log the message on the appropriate channels.
     * @param level the log level for the message, error, warning, etc.
     * @param methodName the name of the method in the class logging the message.
     * @param message the message to log.
     * @throws java.lang.Exception any error.
     */
    public void logMessage(String level, String methodName, String message) throws Exception
    {
        int i;
        String wholeMessage;                                //the whole message
        
        wholeMessage = ((new java.util.Date()).toString() + SEP + level + SEP);
        wholeMessage += (className + SEP + methodName + SEP + message);

        if (canLog(level) && (logChannels != null))
        {
            for (i = 0; i < logChannels.size(); i++)
            {
                ((LogChannel)logChannels.elementAt(i)).writeMessage(wholeMessage);
            }
        }
    }

    /**
     * Debug the message on the appropriate channels.
     * @param level the log level for the message, error, warning, etc.
     * @param methodName the name of the method in the class logging the message.
     * @param message the message to log.
     * @throws java.lang.Exception any error.
     */
    public void debugMessage(String level, String methodName, String message) throws Exception
    {
        int i;
        String wholeMessage;                                //the whole message
        
        wholeMessage = ((new java.util.Date()).toString() + SEP + level + SEP);
        wholeMessage += (className + SEP + methodName + SEP + message);

        if (debug && canDebug(level) && (debugChannels != null))
        {
            for (i = 0; i < debugChannels.size(); i++)
            {
                ((LogChannel)debugChannels.elementAt(i)).writeMessage(wholeMessage);
            }
        }
    }

    /**
     * Log the error on the appropriate channels.
     * @param level the log level for the message, error, warning, etc.
     * @param error the error to log.
     * @throws java.lang.Exception any error.
     */
    public void logError(String level, Exception error) throws Exception
    {
        int i;

        if (canLog(level) && (logChannels != null))
        {
            for (i = 0; i < logChannels.size(); i++)
            {
                ((LogChannel)logChannels.elementAt(i)).writeError(error);
            }
        }
    }

    /**
     * Debug the error on the appropriate channels.
     * @param level the log level for the message, error, warning, etc.
     * @param error the error to debug.
     * @throws java.lang.Exception any error.
     */
    public void debugError(String level, Exception error) throws Exception
    {
        int i;

        if (debug && canDebug(level) && (debugChannels != null))
        {
            for (i = 0; i < debugChannels.size(); i++)
            {
                ((LogChannel)debugChannels.elementAt(i)).writeError(error);
            }
        }
    }

    /**
     * Return true if the logger can log to the specified level.
     * @param messageLevel the level to log to.
     * @return true if can log at this level.
     */
    private boolean canLog(String messageLevel)
    {
        if (logLevel == null) logLevel = CustomLoggerFactory.getLogLevel();
        return LoggerHandler.levelOK(messageLevel, logLevel);
    }

    /**
     * Return true if the logger can debug to the specified level.
     * @param messageLevel the level to log to.
     * @return true if can log at this level.
     */
    private boolean canDebug(String messageLevel)
    {
        if (debugLevel == null) debugLevel = CustomLoggerFactory.getDebugLevel();
        return LoggerHandler.levelOK(messageLevel, debugLevel);
    }
}
